@@ -29,6 +29,7 @@ codigos_tipo_prazo = {}
 codigos_tipo_segmento = {}
 codigos_tipo_fundo = {}
 codigos_tipo_anbima = {}
+codigos_tipo_filtro = {}
 codigos_administradores = {}
 dados_administradores = []
 
@@ -39,6 +40,7 @@ primario_tipo_prazo = {}
 primario_tipo_segmento = {}
 primario_tipo_fundo = {}
 primario_tipo_anbima = {}
+primario_tipo_filtro = {}
 primario_administradores = {}
 
 def parse_parent_content(_content, _exclude):
@@ -222,7 +224,11 @@ class ativo():
         ativo.filtro_variacao_patrimonial_positiva = False
         ativo.filtro_investidor_qualificado = False
         ativo.filtro_dias_sem_pregao = False
-        ativo.filtro = False
+        ativo.filtro_numero_minimo_cotistas = False
+        ativo.filtros_exclusao = 0
+        ativo.filtros_alerta = 0
+        ativo.filtro_exclusao = False
+        ativo.filtro_alerta = False
         ativo.dias_sem_pregao = 0
 
 class provento(object):
@@ -680,11 +686,18 @@ class fii:
             #lap1 = datetime.datetime.now() - lap
             #lap = datetime.datetime.now()
             # browser = webdriver.PhantomJS()
-            browser.get(url)
+            tentativas = 1
+            while tentativas <=100:
+                browser.get(url)
             #lap2 = datetime.datetime.now() - lap
             #lap = datetime.datetime.now()
             # time.sleep(3)
-            html = browser.page_source
+                html = browser.page_source
+                if html.find('Seguir') >= 0:
+                    break
+                tentativas = tentativas + 1
+                print(ticker + 'tentativa ' + str(tentativas))
+
             #lap3 = datetime.datetime.now() - lap
             #lap = datetime.datetime.now()
             soup = BeautifulSoup(html, "lxml")
@@ -716,90 +729,92 @@ class fii:
             #int_pointer = dic_estrutura.get(estrutura[int_contador_itens + 5].strip())
             int_pointer = dic_estrutura.get(ticker)
 
-
-            lst_fiis_interno[int_pointer].seguidores_fiis_com_br = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['SEGUIR']+2]) # primeira fonte
-            lst_fiis_interno[int_pointer].nome = funcs.clean_text(estrutura[dic_estrutura_dinamica['SEGUIR']+4]) # primeira fonte
-            administrador_nome = preferencial(lst_fiis_interno[int_pointer].administrador, funcs.clean_text(estrutura[dic_estrutura_dinamica['ADMINISTRADOR']+2])).upper()
-            administrador_cnpj = funcs.formata_cnpj(funcs.clean_text(estrutura[dic_estrutura_dinamica['ADMINISTRADOR'] + 3]))
-            administrador_fone = funcs.formata_fone(funcs.clean_text(estrutura[dic_estrutura_dinamica['TELEFONE']+2]))
-            administrador_email = funcs.formata_email(funcs.clean_text(estrutura[dic_estrutura_dinamica['EMAIL']+2])).upper()
-            administrador_site = funcs.formata_url(funcs.clean_text(estrutura[dic_estrutura_dinamica['SITE']+2])).upper()
-            try:
-                lst_fiis_interno[int_pointer].codigo_administrador = codigos_administradores[administrador_cnpj]
-            except:
-                codigos_administradores[administrador_cnpj] = len(codigos_administradores)
-                dados_administradores.append([administrador_nome, administrador_cnpj, administrador_fone, administrador_email, administrador_site])
-                lst_fiis_interno[int_pointer].codigo_administrador = codigos_administradores[administrador_cnpj]
-
-            lst_fiis_interno[int_pointer].administrador = administrador_nome
-
-            #lst_fiis_interno[int_pointer].administrador = preferencial(lst_fiis_interno[int_pointer].administrador, funcs.clean_text(estrutura[dic_estrutura_dinamica['ADMINISTRADOR']+2]))
-            #lst_fiis_interno[int_pointer].administrador_cnpj = funcs.formata_cnpj(funcs.clean_text(estrutura[dic_estrutura_dinamica['ADMINISTRADOR']+3]))
-            #lst_fiis_interno[int_pointer].administrador_fone  = funcs.formata_fone(funcs.clean_text(estrutura[dic_estrutura_dinamica['TELEFONE']+2]))
-            #lst_fiis_interno[int_pointer].administrador_email  = funcs.formata_email(funcs.clean_text(estrutura[dic_estrutura_dinamica['EMAIL']+2]))
-            #lst_fiis_interno[int_pointer].administrador_site  = funcs.formata_url(funcs.clean_text(estrutura[dic_estrutura_dinamica['SITE']+2]))
-            lst_fiis_interno[int_pointer].nome_pregao  = funcs.clean_text(estrutura[dic_estrutura_dinamica['NOME NO PREGÃO']+2]) # primeira fonte
-            str_tipo = funcs.clean_text(estrutura[dic_estrutura_dinamica['TIPO DO FII']+2])
-            if str_tipo.upper().find('HÍBRIDO') >= 0:
-                str_tipo = str_tipo.upper().replace('HÍBRIDO ', 'HÍBRIDO: ')
-            if str_tipo.upper().find('INDEFINIDO') >= 0:
-                str_tipo = 'INDEFINIDO: INDETERMINADO'
-            #str_tipo_fundo = str_tipo[:str_tipo.find(':')].upper().replace('N/A', 'INDEFINIDO').replace(' (TIJOLO/PAPEL)', '')
-            str_tipo_fundo = str_tipo[:str_tipo.find(':')].upper()
-            try:
-                lst_fiis_interno[int_pointer].cod_tipo_fundo = preferencial(lst_fiis_interno[int_pointer].cod_tipo_fundo , codigos_tipo_fundo[str_tipo_fundo], ordem = _dict_preferencial['TIPO DE FUNDO'])
-            except:
-                codigos_tipo_fundo[str_tipo_fundo] = len(codigos_tipo_fundo)
-                lst_fiis_interno[int_pointer].cod_tipo_fundo = preferencial(lst_fiis_interno[int_pointer].cod_tipo_fundo, codigos_tipo_fundo[str_tipo_fundo], ordem = _dict_preferencial['TIPO DE FUNDO'])
-            str_segmento = str_tipo[(str_tipo.find(':') + 2):].upper()
-            try:
-                lst_fiis_interno[int_pointer].cod_tipo_segmento = preferencial(lst_fiis_interno[int_pointer].cod_tipo_segmento, codigos_tipo_segmento[str_segmento], código = True, ordem = _dict_preferencial["TIPO DE SEGMENTO"])
-            except:
-                codigos_tipo_segmento[str_segmento] = len(codigos_tipo_segmento)
-                lst_fiis_interno[int_pointer].cod_tipo_segmento = preferencial(lst_fiis_interno[int_pointer].cod_tipo_segmento, codigos_tipo_segmento[str_segmento], código = True, ordem = _dict_preferencial["TIPO DE SEGMENTO"])
-            str_tipo_anbima = funcs.clean_text(estrutura[dic_estrutura_dinamica['TIPO ANBIMA']+2]).upper().replace('VALORE ', 'VALORES ')
-            if str_tipo_anbima.strip() == '':
-                str_tipo_anbima = 'INDEFINIDO'
-            try:
-                lst_fiis_interno[int_pointer].cod_tipo_anbima  = codigos_tipo_anbima[str_tipo_anbima]
-            except:
-                codigos_tipo_anbima[str_tipo_anbima] = len(codigos_tipo_anbima)
-                lst_fiis_interno[int_pointer].cod_tipo_anbima = codigos_tipo_anbima[str_tipo_anbima]
-                #print(funcs.clean_text(estrutura[dic_estrutura_dinamica['TIPO ANBIMA']+2]).upper())
-            str_tipo_anbima = funcs.find_keys_by_val(codigos_tipo_anbima, lst_fiis_interno[int_pointer].cod_tipo_anbima)
-            if str_tipo_anbima.find('ATIVA') >= 0:
-                lst_fiis_interno[int_pointer].cod_tipo_gestao = codigos_tipo_gestao['ATIVA']
-            elif str_tipo_anbima.find('PASSIVA') >= 0:
-                lst_fiis_interno[int_pointer].cod_tipo_gestao = codigos_tipo_gestao['PASSIVA']
+            if tentativas != 1 and html.find('Not Found') != -1:
+                lst_fiis_interno[int_pointer].ativo.faltou_lupa_fiis = true
             else:
-                lst_fiis_interno[int_pointer].cod_tipo_gestao = codigos_tipo_gestao['N/A']
-            data_registro_cvm = funcs.clean_text(estrutura[dic_estrutura_dinamica['REGISTRO CVM']+2])
-            try:
-                lst_fiis_interno[int_pointer].data_registro_cvm  = datetime.datetime.strptime(data_registro_cvm,"%d/%m/%Y")
-            except ValueError:
-                pass
-            lst_fiis_interno[int_pointer].numero_cotas  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['NÚMERO DE COTAS']+2])
-            lst_fiis_interno[int_pointer].numero_cotistas  = preferencial(lst_fiis_interno[int_pointer].numero_cotistas,funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['NÚMERO DE COTISTAS']+2]))
-            lst_fiis_interno[int_pointer].cnpj  = funcs.formata_cnpj(funcs.clean_text(estrutura[dic_estrutura_dinamica['CNPJ']+2]))
-            lst_fiis_interno[int_pointer].notas  = funcs.clean_text(estrutura[int_contador_itens + 31])
-            lst_fiis_interno[int_pointer].taxas = funcs.clean_text(estrutura[dic_estrutura_dinamica['CNPJ']+5])
-            lst_fiis_interno[int_pointer].dy = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['DIVIDEND YIELD']])
-            lst_fiis_interno[int_pointer].dy_projetado = (1 + lst_fiis_interno[int_pointer].dy) ** 12 -1
-            lst_fiis_interno[int_pointer].ultimo_rendimento_brl  = preferencial(lst_fiis_interno[int_pointer].ultimo_rendimento_brl,funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['ÚLTIMO RENDIMENTO']]))
-            lst_fiis_interno[int_pointer].patrimonio_txt  = funcs.clean_text(estrutura[dic_estrutura_dinamica['PATRIMÔNIO LÍQUIDO']])
-            lst_fiis_interno[int_pointer].valor_patrimonial  = preferencial(lst_fiis_interno[int_pointer].valor_patrimonial,funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['VALOR PATRIMONIAL POR COTA']]))
-            lst_fiis_interno[int_pointer].ultimo_preco  = preferencial(lst_fiis_interno[int_pointer].ultimo_preco, funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['MÍN. 52 SEMANAS']-4]))
-            lst_fiis_interno[int_pointer].ultima_valorizacao  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['MÍN. 52 SEMANAS']-3])
+                lst_fiis_interno[int_pointer].seguidores_fiis_com_br = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['SEGUIR']+2]) # primeira fonte
+                lst_fiis_interno[int_pointer].nome = funcs.clean_text(estrutura[dic_estrutura_dinamica['SEGUIR']+4]) # primeira fonte
+                administrador_nome = preferencial(lst_fiis_interno[int_pointer].administrador, funcs.clean_text(estrutura[dic_estrutura_dinamica['ADMINISTRADOR']+2])).upper()
+                administrador_cnpj = funcs.formata_cnpj(funcs.clean_text(estrutura[dic_estrutura_dinamica['ADMINISTRADOR'] + 3]))
+                administrador_fone = funcs.formata_fone(funcs.clean_text(estrutura[dic_estrutura_dinamica['TELEFONE']+2]))
+                administrador_email = funcs.formata_email(funcs.clean_text(estrutura[dic_estrutura_dinamica['EMAIL']+2])).upper()
+                administrador_site = funcs.formata_url(funcs.clean_text(estrutura[dic_estrutura_dinamica['SITE']+2])).upper()
+                try:
+                    lst_fiis_interno[int_pointer].codigo_administrador = codigos_administradores[administrador_cnpj]
+                except:
+                    codigos_administradores[administrador_cnpj] = len(codigos_administradores)
+                    dados_administradores.append([administrador_nome, administrador_cnpj, administrador_fone, administrador_email, administrador_site])
+                    lst_fiis_interno[int_pointer].codigo_administrador = codigos_administradores[administrador_cnpj]
 
-            lst_fiis_interno[int_pointer].min_52_semanas  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['MÍN. 52 SEMANAS']])
-            lst_fiis_interno[int_pointer].max_52_semanas  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['MÁX. 52 SEMANAS']])
-            lst_fiis_interno[int_pointer].variacao_12_meses  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['VALORIZAÇÃO (12 MESES)']-1]) / 100
-            lst_fiis_interno[int_pointer].execucao_curta = True
+                lst_fiis_interno[int_pointer].administrador = administrador_nome
 
-            # print('Ativo: %s, Lap: %s, Total: %s' % (lst_fiis_interno[int_pointer].ticker, str(datetime.datetime.now()-lap), str(datetime.datetime.now()-start)))
-            # browser.close()
-            # browser.quit() # teste
-            #lap = datetime.datetime.now()
+                #lst_fiis_interno[int_pointer].administrador = preferencial(lst_fiis_interno[int_pointer].administrador, funcs.clean_text(estrutura[dic_estrutura_dinamica['ADMINISTRADOR']+2]))
+                #lst_fiis_interno[int_pointer].administrador_cnpj = funcs.formata_cnpj(funcs.clean_text(estrutura[dic_estrutura_dinamica['ADMINISTRADOR']+3]))
+                #lst_fiis_interno[int_pointer].administrador_fone  = funcs.formata_fone(funcs.clean_text(estrutura[dic_estrutura_dinamica['TELEFONE']+2]))
+                #lst_fiis_interno[int_pointer].administrador_email  = funcs.formata_email(funcs.clean_text(estrutura[dic_estrutura_dinamica['EMAIL']+2]))
+                #lst_fiis_interno[int_pointer].administrador_site  = funcs.formata_url(funcs.clean_text(estrutura[dic_estrutura_dinamica['SITE']+2]))
+                lst_fiis_interno[int_pointer].nome_pregao  = funcs.clean_text(estrutura[dic_estrutura_dinamica['NOME NO PREGÃO']+2]) # primeira fonte
+                str_tipo = funcs.clean_text(estrutura[dic_estrutura_dinamica['TIPO DO FII']+2])
+                if str_tipo.upper().find('HÍBRIDO') >= 0:
+                    str_tipo = str_tipo.upper().replace('HÍBRIDO ', 'HÍBRIDO: ')
+                if str_tipo.upper().find('INDEFINIDO') >= 0:
+                    str_tipo = 'INDEFINIDO: INDETERMINADO'
+                #str_tipo_fundo = str_tipo[:str_tipo.find(':')].upper().replace('N/A', 'INDEFINIDO').replace(' (TIJOLO/PAPEL)', '')
+                str_tipo_fundo = str_tipo[:str_tipo.find(':')].upper()
+                try:
+                    lst_fiis_interno[int_pointer].cod_tipo_fundo = preferencial(lst_fiis_interno[int_pointer].cod_tipo_fundo , codigos_tipo_fundo[str_tipo_fundo], ordem = _dict_preferencial['TIPO DE FUNDO'])
+                except:
+                    codigos_tipo_fundo[str_tipo_fundo] = len(codigos_tipo_fundo)
+                    lst_fiis_interno[int_pointer].cod_tipo_fundo = preferencial(lst_fiis_interno[int_pointer].cod_tipo_fundo, codigos_tipo_fundo[str_tipo_fundo], ordem = _dict_preferencial['TIPO DE FUNDO'])
+                str_segmento = str_tipo[(str_tipo.find(':') + 2):].upper()
+                try:
+                    lst_fiis_interno[int_pointer].cod_tipo_segmento = preferencial(lst_fiis_interno[int_pointer].cod_tipo_segmento, codigos_tipo_segmento[str_segmento], código = True, ordem = _dict_preferencial["TIPO DE SEGMENTO"])
+                except:
+                    codigos_tipo_segmento[str_segmento] = len(codigos_tipo_segmento)
+                    lst_fiis_interno[int_pointer].cod_tipo_segmento = preferencial(lst_fiis_interno[int_pointer].cod_tipo_segmento, codigos_tipo_segmento[str_segmento], código = True, ordem = _dict_preferencial["TIPO DE SEGMENTO"])
+                str_tipo_anbima = funcs.clean_text(estrutura[dic_estrutura_dinamica['TIPO ANBIMA']+2]).upper().replace('VALORE ', 'VALORES ')
+                if str_tipo_anbima.strip() == '':
+                    str_tipo_anbima = 'INDEFINIDO'
+                try:
+                    lst_fiis_interno[int_pointer].cod_tipo_anbima  = codigos_tipo_anbima[str_tipo_anbima]
+                except:
+                    codigos_tipo_anbima[str_tipo_anbima] = len(codigos_tipo_anbima)
+                    lst_fiis_interno[int_pointer].cod_tipo_anbima = codigos_tipo_anbima[str_tipo_anbima]
+                    #print(funcs.clean_text(estrutura[dic_estrutura_dinamica['TIPO ANBIMA']+2]).upper())
+                str_tipo_anbima = funcs.find_keys_by_val(codigos_tipo_anbima, lst_fiis_interno[int_pointer].cod_tipo_anbima)
+                if str_tipo_anbima.find('ATIVA') >= 0:
+                    lst_fiis_interno[int_pointer].cod_tipo_gestao = codigos_tipo_gestao['ATIVA']
+                elif str_tipo_anbima.find('PASSIVA') >= 0:
+                    lst_fiis_interno[int_pointer].cod_tipo_gestao = codigos_tipo_gestao['PASSIVA']
+                else:
+                    lst_fiis_interno[int_pointer].cod_tipo_gestao = codigos_tipo_gestao['N/A']
+                data_registro_cvm = funcs.clean_text(estrutura[dic_estrutura_dinamica['REGISTRO CVM']+2])
+                try:
+                    lst_fiis_interno[int_pointer].data_registro_cvm  = datetime.datetime.strptime(data_registro_cvm,"%d/%m/%Y")
+                except ValueError:
+                    pass
+                lst_fiis_interno[int_pointer].numero_cotas  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['NÚMERO DE COTAS']+2])
+                lst_fiis_interno[int_pointer].numero_cotistas  = preferencial(lst_fiis_interno[int_pointer].numero_cotistas,funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['NÚMERO DE COTISTAS']+2]))
+                lst_fiis_interno[int_pointer].cnpj  = funcs.formata_cnpj(funcs.clean_text(estrutura[dic_estrutura_dinamica['CNPJ']+2]))
+                lst_fiis_interno[int_pointer].notas  = funcs.clean_text(estrutura[int_contador_itens + 31])
+                lst_fiis_interno[int_pointer].taxas = funcs.clean_text(estrutura[dic_estrutura_dinamica['CNPJ']+5])
+                lst_fiis_interno[int_pointer].dy = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['DIVIDEND YIELD']])
+                lst_fiis_interno[int_pointer].dy_projetado = (1 + lst_fiis_interno[int_pointer].dy) ** 12 -1
+                lst_fiis_interno[int_pointer].ultimo_rendimento_brl  = preferencial(lst_fiis_interno[int_pointer].ultimo_rendimento_brl,funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['ÚLTIMO RENDIMENTO']]))
+                lst_fiis_interno[int_pointer].patrimonio_txt  = funcs.clean_text(estrutura[dic_estrutura_dinamica['PATRIMÔNIO LÍQUIDO']])
+                lst_fiis_interno[int_pointer].valor_patrimonial  = preferencial(lst_fiis_interno[int_pointer].valor_patrimonial,funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['VALOR PATRIMONIAL POR COTA']]))
+                lst_fiis_interno[int_pointer].ultimo_preco  = preferencial(lst_fiis_interno[int_pointer].ultimo_preco, funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['MÍN. 52 SEMANAS']-4]))
+                lst_fiis_interno[int_pointer].ultima_valorizacao  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['MÍN. 52 SEMANAS']-3])
+
+                lst_fiis_interno[int_pointer].min_52_semanas  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['MÍN. 52 SEMANAS']])
+                lst_fiis_interno[int_pointer].max_52_semanas  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['MÁX. 52 SEMANAS']])
+                lst_fiis_interno[int_pointer].variacao_12_meses  = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['VALORIZAÇÃO (12 MESES)']-1]) / 100
+                lst_fiis_interno[int_pointer].execucao_curta = True
+
+                # print('Ativo: %s, Lap: %s, Total: %s' % (lst_fiis_interno[int_pointer].ticker, str(datetime.datetime.now()-lap), str(datetime.datetime.now()-start)))
+                # browser.close()
+                # browser.quit() # teste
+                #lap = datetime.datetime.now()
             pbar.update(1)
         pbar.close()
 
@@ -900,8 +915,8 @@ class fii:
                 try:
                     str_mes_data_constituicao = str_data_constituicao[str_data_constituicao.find(' DE ')+4:str_data_constituicao.rfind(' DE ')]
                     str_data_constituicao = str_data_constituicao.replace(' DE ' + str_mes_data_constituicao + ' DE ', '/' + dic_meses[str_mes_data_constituicao]+'/')
-                    lst_fiis_interno[int_pointer].data_constituicao = datetime.datetime.stpftime(str_data_constituicao,"%d/%m/%Y")
-                except ValueError:
+                    lst_fiis_interno[int_pointer].data_constituicao = datetime.datetime.strptime(str_data_constituicao,"%d/%m/%Y")
+                except (ValueError, KeyError):
                     pass
                 lst_fiis_interno[int_pointer].cotas_emitidas = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['COTAS EMITIDAS']+2])
                 lst_fiis_interno[int_pointer].patrimonio_inicial = funcs.num_ptb2us(estrutura[dic_estrutura_dinamica['PATRIMÔNIO INICIAL']+2])
@@ -1009,8 +1024,8 @@ class fii:
         #for line in lst_tempos:
         #    print(line)
         _ultimo_pregao_ibov = cotacoes2.ultimo_pregao('ibov')[0]
-        #pbar = tqdm(total=len(lst_fiis_interno) + 1, unit=' ativos')
-        for item in tqdm(lst_fiis_interno, leave=True, unit=' ativos'):
+        pbar = tqdm(total=len(lst_fiis_interno) + 1, unit=' ativos', leave=True)
+        for item in lst_fiis_interno:
             if item.execucao_curta == True:
                 pbar.set_description('Carregando dias sem pregao - ' + item.ticker)
                 try:
@@ -1019,6 +1034,6 @@ class fii:
                 except:
                     item.dias_sem_pregao = 9999
                 _lst_fiis.append(item)
-                #pbar.update(1)
+                pbar.update(1)
 
         #_lst_fiis.append(lst_fiis_interno)
